#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Adapted from Conbagui's tutorial found here: https://www.deviantart.com/conbagui/art/Create-Christmas-Candles-and-Curtains-with-Gimp-2-824241717

# author: Pocholo
# date: 12/18/21

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>. 


from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_christmas_candles_background(curtPattern, canholPattern, candColor):
           
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
	
    #Create new canvas 
    width = 1600
    height = 1200
    img = pdb.gimp_image_new(width, height, RGB)
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Curtain Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 0)
    pdb.gimp_context_set_pattern(curtPattern)
    pdb.gimp_drawable_fill(backLayer, FILL_PATTERN)
    
	#Image process
    pdb.gimp_display_new(img)
    
    #Curtain fullness effect
    fullnessLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Curtain Fullness", 92, LAYER_MODE_MULTIPLY)
    pdb.gimp_image_add_layer(img, fullnessLayer, 0)
    pdb.gimp_drawable_fill(fullnessLayer, FILL_WHITE)
    pdb.plug_in_sinus(img, fullnessLayer, 
									  35,   #xscale 
									   1,   #yscale 
								    1.00,   #complex 
									  36,   #seed 
								    True,   #tiling
								    True,   #perturb 
									   2,   #colors - choose
							   (0, 0, 0),   #col1 
						  (255, 255, 255),   #col2 
									   1,   #alpha1 
									   1,   #alpha2
									   2,   #blend 
									   0)   #blend_power
    pdb.gimp_drawable_offset(fullnessLayer, TRUE, 2, 55, 0)
    backLayer = pdb.gimp_image_merge_down(img, fullnessLayer, CLIP_TO_IMAGE)
    
    #Candles holder
    holderLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Candles Holder", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, holderLayer, 0)
    pdb.gimp_drawable_fill(holderLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(2)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.1)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_paintbrush_default(holderLayer, 20, [800, 1200,
                                                  175, 1200,
                                                  175, 1100,
                                                  350, 1100,
                                                  350, 1000,
                                                  525, 1000,
                                                  525, 900,
                                                  700, 900,
                                                  700, 800,
                                                  800, 800])
    
    non_empty = pdb.gimp_edit_copy(holderLayer)
    holderLayer2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Layer", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, holderLayer2, 0)
    pdb.gimp_drawable_fill(holderLayer2, FILL_TRANSPARENT)
    holderLayer2 = pdb.gimp_edit_paste(holderLayer2, FALSE)
    pdb.gimp_floating_sel_anchor(holderLayer2)
    holderLayer2 = pdb.gimp_image_get_layer_by_name(img, "Layer")
    holderLayer2 = pdb.gimp_flip(holderLayer2, ORIENTATION_HORIZONTAL)
    holderLayer = pdb.gimp_image_merge_down(img, holderLayer2, CLIP_TO_IMAGE)
    pdb.gimp_context_set_sample_transparent(TRUE)
    pdb.gimp_image_select_contiguous_color(img, 2, holderLayer, 780, 1050)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_edit_bucket_fill(holderLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_context_set_pattern(canholPattern)
    pdb.gimp_edit_bucket_fill(holderLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_shrink(img, 5)
    pdb.gimp_selection_border(img, 5)
    pdb.gimp_context_set_gradient("Golden")
    pdb.gimp_context_set_gradient_repeat_mode(REPEAT_SAWTOOTH)
    pdb.gimp_drawable_edit_gradient_fill(holderLayer, 0, 0, TRUE, 3, 0.20, TRUE, 0, 1200, 175, 1200)
    pdb.gimp_selection_none(img)
    
    #Middle candle
    width = 150
    height = 600
    mcandleLayer = pdb.gimp_layer_new(img, width, height, RGBA_IMAGE, "Middle candle", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, mcandleLayer, 1)
    pdb.gimp_drawable_fill(mcandleLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground(candColor)
    pdb.gimp_drawable_fill(mcandleLayer, FILL_FOREGROUND)
    pdb.gimp_layer_translate(mcandleLayer, 725, 212)
    #using map to create the candle shape
    pdb.plug_in_map_object(img, mcandleLayer, 
										   3,    # Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
							   0.5, 0.5, 2.0,    # viewpoint
							   0.5, 0.5, 0.0,    # object pos
							   1.0, 0.0, 0.0,    # first axis
							   0.0, 1.0, 0.0,    # 2nd axis
							   0.0, 0.0, 0.0,    # axis rotation
									 	   1,    # Type of lightsource (0=point,1=directional,2=none)
					         (255, 255, 255),    # Lightsource color (r,g,b)
					         0.40, -0.5, 2.0,    # light position
						   -0.20, -0.40, 1.0,    # light direction
			    0.60, 1.00, 0.50, 0.20, 27.0,    # material (amb, diff, refl, spec, high)
								        True,    # antialias
									   False,    # tile
									   False,    # new image
								        True,    # transparency
									    0.25,    # Sphere/cylinder radius (only used when maptype=1 or 3)
							   0.5, 0.5, 0.5,    # box size
								        1.00,    # unused parameter Cylinder length
									    None,    # Box front face (set these to -1 if not used)
									    None,    # Box back face
									    None,    # Box top face									
									    None,    # Box bottom face
									    None,    # Box left face
									    None,    # Box right face
									    None,    # Cylinder top face (unused parameter)
									    None)    # Cylinder bottom face (unused parameter)
    
    #First candle to the left
    non_empty = pdb.gimp_edit_copy(mcandleLayer)
    leftCandle1 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Left #1", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, leftCandle1, 1)
    pdb.gimp_drawable_fill(leftCandle1, FILL_TRANSPARENT)
    leftCandle1 = pdb.gimp_edit_paste(leftCandle1, FALSE)
    pdb.gimp_layer_translate(leftCandle1, -180, 13)
    pdb.gimp_floating_sel_anchor(leftCandle1)
    leftCandle1 = pdb.gimp_image_get_layer_by_name(img, "Left #1")
    pdb.gimp_layer_resize_to_image_size(leftCandle1)
    
    #Second candle to the left
    non_empty = pdb.gimp_edit_copy(leftCandle1)
    leftCandle2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Left #2", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, leftCandle2, 1)
    pdb.gimp_drawable_fill(leftCandle2, FILL_TRANSPARENT)
    leftCandle2 = pdb.gimp_edit_paste(leftCandle2, FALSE)
    pdb.gimp_layer_translate(leftCandle2, -173, 103)
    pdb.gimp_floating_sel_anchor(leftCandle2)
    leftCandle2 = pdb.gimp_image_get_layer_by_name(img, "Left #2")
    pdb.gimp_layer_resize_to_image_size(leftCandle2)
    
    #Third candle to the left
    non_empty = pdb.gimp_edit_copy(leftCandle2)
    leftCandle3 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Left #3", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, leftCandle3, 1)
    pdb.gimp_drawable_fill(leftCandle3, FILL_TRANSPARENT)
    leftCandle3 = pdb.gimp_edit_paste(leftCandle3, FALSE)
    pdb.gimp_layer_translate(leftCandle3, -180, 102)
    pdb.gimp_floating_sel_anchor(leftCandle3)
    leftCandle3 = pdb.gimp_image_get_layer_by_name(img, "Left #3")
    pdb.gimp_layer_resize_to_image_size(leftCandle3)
    
    #Create a copy and flip to make six candles
    pdb.gimp_item_set_visible(backLayer, FALSE)
    pdb.gimp_item_set_visible(holderLayer, FALSE)
    pdb.gimp_item_set_visible(mcandleLayer, FALSE)
    candleLayer = pdb.gimp_image_merge_visible_layers(img, CLIP_TO_IMAGE)
    pdb.gimp_item_set_name(candleLayer, "Candles")
    pdb.gimp_item_set_visible(mcandleLayer, TRUE)
    non_empty = pdb.gimp_edit_copy(candleLayer)
    candleLayer2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Layer", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, candleLayer2, 1)
    pdb.gimp_drawable_fill(candleLayer2, FILL_TRANSPARENT)
    candleLayer2 = pdb.gimp_edit_paste(candleLayer2, FALSE)
    pdb.gimp_floating_sel_anchor(candleLayer2)
    candleLayer2 = pdb.gimp_image_get_layer_by_name(img, "Layer")
    candleLayer2 = pdb.gimp_flip(candleLayer2, ORIENTATION_HORIZONTAL)
    candleLayer = pdb.gimp_image_merge_down(img, candleLayer2, CLIP_TO_IMAGE)
    pdb.gimp_item_set_visible(backLayer, TRUE)
    pdb.gimp_item_set_visible(holderLayer, TRUE)
    candleLayer = pdb.gimp_image_get_layer_by_name(img, "Candles")
    mcandleLayer = pdb.gimp_image_merge_down(img, candleLayer, CLIP_TO_IMAGE)
    pdb.gimp_item_set_name(mcandleLayer, "Candles")
	
    #Creating the wick
    wickLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Candles wick", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, wickLayer, 2)
    pdb.gimp_drawable_fill(wickLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_context_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(7)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.1)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    
    #1
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_paintbrush_default(wickLayer, 4, [265, 526, 265, 498])
    pdb.gimp_image_select_item(img, 2, wickLayer)
    pdb.gimp_image_select_ellipse(img, 3, 230, 460, 70, 80)
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(wickLayer, 0, 0, TRUE, 3, 0.20, TRUE, 265, 530, 265, 494)
    pdb.gimp_selection_none(img)
    #2
    pdb.gimp_paintbrush_default(wickLayer, 4, [445, 422, 445, 398])
    pdb.gimp_image_select_item(img, 2, wickLayer)
    pdb.gimp_image_select_ellipse(img, 3, 410, 360, 70, 80)
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(wickLayer, 0, 0, TRUE, 3, 0.20, TRUE, 445, 426, 445, 394)
    pdb.gimp_selection_none(img)
    #3
    pdb.gimp_paintbrush_default(wickLayer, 4, [618, 319, 618, 296])
    pdb.gimp_image_select_item(img, 2, wickLayer)
    pdb.gimp_image_select_ellipse(img, 3, 583, 298, 70, 80)
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(wickLayer, 0, 0, TRUE, 3, 0.20, TRUE, 618, 323, 618, 293)
    pdb.gimp_selection_none(img)
    #4
    pdb.gimp_paintbrush_default(wickLayer, 4, [800, 218, 800, 192])
    pdb.gimp_image_select_item(img, 2, wickLayer)
    pdb.gimp_image_select_ellipse(img, 3, 765, 156, 70, 80)
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(wickLayer, 0, 0, TRUE, 3, 0.20, TRUE, 800, 220, 800, 190)
    pdb.gimp_selection_none(img)
    #5
    pdb.gimp_paintbrush_default(wickLayer, 4, [981, 319, 981, 296])
    pdb.gimp_image_select_item(img, 2, wickLayer)
    pdb.gimp_image_select_ellipse(img, 3, 945, 257, 70, 80)
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(wickLayer, 0, 0, TRUE, 3, 0.20, TRUE, 981, 323, 981, 293)
    pdb.gimp_selection_none(img)
    #6
    pdb.gimp_paintbrush_default(wickLayer, 4, [1154, 422, 1154, 398])
    pdb.gimp_image_select_item(img, 2, wickLayer)
    pdb.gimp_image_select_ellipse(img, 3, 1120, 361, 70, 80)
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(wickLayer, 0, 0, TRUE, 3, 0.20, TRUE, 1154, 426, 1154, 395)
    pdb.gimp_selection_none(img)
    #7
    pdb.gimp_paintbrush_default(wickLayer, 4, [1332, 526, 1332, 498])
    pdb.gimp_image_select_item(img, 2, wickLayer)
    pdb.gimp_image_select_ellipse(img, 3, 1296, 460, 70, 80)
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(wickLayer, 0, 0, TRUE, 3, 0.20, TRUE, 1332, 530, 1332, 495)
    pdb.gimp_selection_none(img)
    pdb.plug_in_ripple(img, wickLayer, 40, 2, 0, 1, 1, TRUE, TRUE)
	
    #Flames
    flameLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Candles flame", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, flameLayer, 3)
    pdb.gimp_drawable_fill(flameLayer, FILL_TRANSPARENT)
    pdb.gimp_selection_none(img)
    #1
    pdb.gimp_image_select_ellipse(img, 2, 253, 446, 25, 60)
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_context_set_gradient_reverse(FALSE)
    pdb.gimp_drawable_edit_gradient_fill(flameLayer, 0, 0, TRUE, 3, 0.20, TRUE, 265, 446, 265, 504)
    pdb.gimp_selection_none(img)
	#2
    pdb.gimp_image_select_ellipse(img, 2, 433, 347, 25, 60)
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_drawable_edit_gradient_fill(flameLayer, 0, 0, TRUE, 3, 0.20, TRUE, 446, 347, 446, 408)
    pdb.gimp_selection_none(img)
	#3
    pdb.gimp_image_select_ellipse(img, 2, 605, 248, 25, 60)
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_drawable_edit_gradient_fill(flameLayer, 0, 0, TRUE, 3, 0.20, TRUE, 618, 246, 618, 308)
    pdb.gimp_selection_none(img)
	#4
    pdb.gimp_image_select_ellipse(img, 2, 788, 147, 25, 60)
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_drawable_edit_gradient_fill(flameLayer, 0, 0, TRUE, 3, 0.20, TRUE, 800, 147, 800, 207)
    pdb.gimp_selection_none(img)
	#5
    pdb.gimp_image_select_ellipse(img, 2, 969, 250, 25, 60)
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_drawable_edit_gradient_fill(flameLayer, 0, 0, TRUE, 3, 0.20, TRUE, 982, 250, 982, 310)
    pdb.gimp_selection_none(img)
	#6
    pdb.gimp_image_select_ellipse(img, 2, 1140, 353, 25, 60)
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_drawable_edit_gradient_fill(flameLayer, 0, 0, TRUE, 3, 0.20, TRUE, 1152, 353, 1152, 414)
    pdb.gimp_selection_none(img)
	#7
    pdb.gimp_image_select_ellipse(img, 2, 1320, 454, 25, 60)
    pdb.gimp_context_set_gradient("Candle flame")
    pdb.gimp_drawable_edit_gradient_fill(flameLayer, 0, 0, TRUE, 3, 0.20, TRUE, 1332, 454, 1332, 514)
    pdb.gimp_selection_none(img)
    pdb.plug_in_ripple(img, flameLayer, 60, 1, 0, 1, 1, TRUE, TRUE)
	
	#Candle light  
    lightLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Candles light", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, lightLayer, 1)
    pdb.gimp_drawable_fill(lightLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_item(img, 2, flameLayer)
    pdb.gimp_selection_grow(img, 10)
    pdb.gimp_context_set_foreground((255, 224, 145))
    pdb.gimp_edit_bucket_fill(lightLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, lightLayer, 100, 100, 1)   #method=1

    #Make active the bottom layer and done
    pdb.gimp_image_set_active_layer(img, backLayer)
    #Set the defaults
    pdb.gimp_context_set_defaults()
     
register(
    "pm_christmas_candles_background",
    "Creates a Christmas candles background",
    "Creates a Christmas candles background",
    "Pocholo",
    "Pocholo",
    "2021",
    "Christmas Candles Background",
    "",
    [
    (PF_PATTERN, "curtPattern", "Curtain Design", None),
    (PF_PATTERN, "canholPattern", "Candle holder Design", None),
    (PF_COLOR, "candColor", "Candle color", (140, 0, 0)),
    ],
    [],
    pm_christmas_candles_background, menu="<Image>/Pocholo-scripts/Christmas Candles Background",
    domain=("gimp20", gimp.locale_directory))
main()				